@@ -25,8 +25,8 @@ class BaseTV(object):
                                 'com.ellation.vrv': ['audio_state'],
                                 'com.hulu.plus': [{'playing': {'wake_lock_size' : 4}},
                                                   {'paused': {'wake_lock_size': 2}}],
-                                'com.plexapp.android': [{'playing': {'media_session_state': 3, 'wake_lock_size': 3}},
-                                                        {'paused': {'media_session_state': 3, 'wake_lock_size': 1}},
+                                'com.plexapp.android': [{'paused': {'media_session_state': 3, 'wake_lock_size': 1}},
+                                                        {'playing': {'media_session_state': 3}},
                                                         'standby']}
 
     The keys are app IDs, and the values are lists of rules that are evaluated in order.
@@ -51,7 +51,9 @@ class BaseTV(object):
     Parameters
     ----------
     host : str
-        The address of the device in the format ``<ip address>:<host>``
+        The address of the device; may be an IP address or a host name
+    port : int
+        The device port to which we are connecting (default is 5555)
     adbkey : str
         The path to the ``adbkey`` file for ADB authentication
     adb_server_ip : str
@@ -60,11 +62,14 @@ class BaseTV(object):
         The port for the ADB server
     state_detection_rules : dict, None
         A dictionary of rules for determining the state (see above)
+    auth_timeout_s : float
+        Authentication timeout (in seconds)
 
     """
 
-    def __init__(self, host, adbkey='', adb_server_ip='', adb_server_port=5037, state_detection_rules=None):
+    def __init__(self, host, port=5555, adbkey='', adb_server_ip='', adb_server_port=5037, state_detection_rules=None, auth_timeout_s=constants.DEFAULT_AUTH_TIMEOUT_S):
         self.host = host
+        self.port = int(port)
         self.adbkey = adbkey
         self.adb_server_ip = adb_server_ip
         self.adb_server_port = adb_server_port
@@ -83,13 +88,13 @@ class BaseTV(object):
         # the handler for ADB commands
         if not adb_server_ip:
             # python-adb
-            self.adb = ADBPython(host, adbkey)
+            self._adb = ADBPython(host, port, adbkey)
         else:
             # pure-python-adb
-            self.adb = ADBServer(host, adb_server_ip, adb_server_port)
+            self._adb = ADBServer(host, port, adb_server_ip, adb_server_port)
 
         # establish the ADB connection
-        self.adb.connect()
+        self.adb_connect(auth_timeout_s=auth_timeout_s)
 
         # get device properties
         self.device_properties = self.get_device_properties()
@@ -116,15 +121,63 @@ class BaseTV(object):
             The response from the device, if there is a response
 
         """
-        return self.adb.shell(cmd)
+        return self._adb.shell(cmd)
 
-    def connect(self, always_log_errors=True):
+    def adb_pull(self, local_path, device_path):
+        """Pull a file from the device.
+
+        This calls :py:meth:`androidtv.adb_manager.ADBPython.pull` or :py:meth:`androidtv.adb_manager.ADBServer.pull`,
+        depending on whether the Python ADB implementation or an ADB server is used for communicating with the device.
+
+        Parameters
+        ----------
+        local_path : str
+            The path where the file will be saved
+        device_path : str
+            The file on the device that will be pulled
+
+        """
+        return self._adb.pull(local_path, device_path)
+
+    def adb_push(self, local_path, device_path):
+        """Push a file to the device.
+
+        This calls :py:meth:`androidtv.adb_manager.ADBPython.push` or :py:meth:`androidtv.adb_manager.ADBServer.push`,
+        depending on whether the Python ADB implementation or an ADB server is used for communicating with the device.
+
+        Parameters
+        ----------
+        local_path : str
+            The file that will be pushed to the device
+        device_path : str
+            The path where the file will be saved on the device
+
+        """
+        return self._adb.push(local_path, device_path)
+
+    def adb_screencap(self):
+        """Take a screencap.
+
+        This calls :py:meth:`androidtv.adb_manager.ADBPython.screencap` or :py:meth:`androidtv.adb_manager.ADBServer.screencap`,
+        depending on whether the Python ADB implementation or an ADB server is used for communicating with the device.
+
+        Returns
+        -------
+        bytes
+            The screencap as a binary .png image
+
+        """
+        return self._adb.screencap()
+
+    def adb_connect(self, always_log_errors=True, auth_timeout_s=constants.DEFAULT_AUTH_TIMEOUT_S):
         """Connect to an Android TV / Fire TV device.
 
         Parameters
         ----------
         always_log_errors : bool
             If True, errors will always be logged; otherwise, errors will only be logged on the first failed reconnect attempt
+        auth_timeout_s : float
+            Authentication timeout (in seconds)
 
         Returns
         -------
@@ -132,18 +185,18 @@ class BaseTV(object):
             Whether or not the connection was successfully established and the device is available
 
         """
-        return self.adb.connect(always_log_errors)
+        if isinstance(self._adb, ADBPython):
+            return self._adb.connect(always_log_errors, auth_timeout_s)
+        return self._adb.connect(always_log_errors)
 
-    def _key(self, key):
-        """Send a key event to device.
+    def adb_close(self):
+        """Close the ADB connection.
 
-        Parameters
-        ----------
-        key : str, int
-            The Key constant
+        This only works for the Python ADB implementation (see :meth:`androidtv.adb_manager.ADBPython.close`).
+        For the ADB server approach, this doesn't do anything (see :meth:`androidtv.adb_manager.ADBServer.close`).
 
         """
-        self.adb.shell('input keyevent {0}'.format(key))
+        self._adb.close()
 
     # ======================================================================= #
     #                                                                         #
@@ -159,12 +212,14 @@ class BaseTV(object):
             A dictionary with keys ``'wifimac'``, ``'ethmac'``, ``'serialno'``, ``'manufacturer'``, ``'model'``, and ``'sw_version'``
 
         """
-        properties = self.adb.shell(constants.CMD_MANUFACTURER + " && " +
-                                    constants.CMD_MODEL + " && " +
-                                    constants.CMD_SERIALNO + " && " +
-                                    constants.CMD_VERSION + " && " +
-                                    constants.CMD_MAC_WLAN0 + " && " +
-                                    constants.CMD_MAC_ETH0)
+        properties = self._adb.shell(constants.CMD_MANUFACTURER + " && " +
+                                     constants.CMD_MODEL + " && " +
+                                     constants.CMD_SERIALNO + " && " +
+                                     constants.CMD_VERSION + " && " +
+                                     constants.CMD_MAC_WLAN0 + " && " +
+                                     constants.CMD_MAC_ETH0)
+
+        _LOGGER.debug("%s:%d `get_device_properties` response: %s", self.host, self.port, properties)
 
         if not properties:
             return {}
@@ -176,7 +231,7 @@ class BaseTV(object):
         manufacturer, model, serialno, version, mac_wlan0_output, mac_eth0_output = lines
 
         if not serialno.strip():
-            _LOGGER.warning("Could not obtain serialno for %s, got: '%s'", self.host, serialno)
+            _LOGGER.warning("Could not obtain serialno for %s:%d, got: '%s'", self.host, self.port, serialno)
             serialno = None
 
         mac_wlan0_matches = re.findall(constants.MAC_REGEX_PATTERN, mac_wlan0_output)
@@ -301,6 +356,20 @@ class BaseTV(object):
     #                               Properties                                #
     #                                                                         #
     # ======================================================================= #
+    @property
+    def audio_output_device(self):
+        """Get the current audio playback device.
+
+        Returns
+        -------
+        str, None
+            The current audio playback device, or ``None`` if it could not be determined
+
+        """
+        stream_music = self._get_stream_music()
+
+        return self._audio_output_device(stream_music)
+
     @property
     def audio_state(self):
         """Check if audio is playing, paused, or idle.
@@ -311,7 +380,7 @@ class BaseTV(object):
             The audio state, as determined from the ADB shell command :py:const:`androidtv.constants.CMD_AUDIO_STATE`, or ``None`` if it could not be determined
 
         """
-        audio_state_response = self.adb.shell(constants.CMD_AUDIO_STATE)
+        audio_state_response = self._adb.shell(constants.CMD_AUDIO_STATE)
         return self._audio_state(audio_state_response)
 
     @property
@@ -324,7 +393,7 @@ class BaseTV(object):
             Whether or not the ADB connection is intact
 
         """
-        return self.adb.available
+        return self._adb.available
 
     @property
     def awake(self):
@@ -336,7 +405,7 @@ class BaseTV(object):
             Whether or not the device is awake (screensaver is not running)
 
         """
-        return self.adb.shell(constants.CMD_AWAKE + constants.CMD_SUCCESS1_FAILURE0) == '1'
+        return self._adb.shell(constants.CMD_AWAKE + constants.CMD_SUCCESS1_FAILURE0) == '1'
 
     @property
     def current_app(self):
@@ -348,24 +417,10 @@ class BaseTV(object):
             The ID of the current app, or ``None`` if it could not be determined
 
         """
-        current_app_response = self.adb.shell(constants.CMD_CURRENT_APP)
+        current_app_response = self._adb.shell(constants.CMD_CURRENT_APP)
 
         return self._current_app(current_app_response)
 
-    @property
-    def device(self):
-        """Get the current playback device.
-
-        Returns
-        -------
-        str, None
-            The current playback device, or ``None`` if it could not be determined
-
-        """
-        stream_music = self._get_stream_music()
-
-        return self._device(stream_music)
-
     @property
     def is_volume_muted(self):
         """Whether or not the volume is muted.
@@ -390,26 +445,12 @@ class BaseTV(object):
             The state from the output of the ADB shell command ``dumpsys media_session``, or ``None`` if it could not be determined
 
         """
-        media_session_state_response = self.adb.shell(constants.CMD_MEDIA_SESSION_STATE_FULL)
+        media_session_state_response = self._adb.shell(constants.CMD_MEDIA_SESSION_STATE_FULL)
 
         _, media_session_state = self._current_app_media_session_state(media_session_state_response)
 
         return media_session_state
 
-    @property
-    def running_apps(self):
-        """Return a list of running user applications.
-
-        Returns
-        -------
-        list
-            A list of the running apps
-
-        """
-        running_apps_response = self.adb.shell(constants.CMD_RUNNING_APPS)
-
-        return self._running_apps(running_apps_response)
-
     @property
     def screen_on(self):
         """Check if the screen is on.
@@ -420,7 +461,7 @@ class BaseTV(object):
             Whether or not the device is on
 
         """
-        return self.adb.shell(constants.CMD_SCREEN_ON + constants.CMD_SUCCESS1_FAILURE0) == '1'
+        return self._adb.shell(constants.CMD_SCREEN_ON + constants.CMD_SUCCESS1_FAILURE0) == '1'
 
     @property
     def volume(self):
@@ -433,9 +474,9 @@ class BaseTV(object):
 
         """
         stream_music = self._get_stream_music()
-        device = self._device(stream_music)
+        audio_output_device = self._audio_output_device(stream_music)
 
-        return self._volume(stream_music, device)
+        return self._volume(stream_music, audio_output_device)
 
     @property
     def volume_level(self):
@@ -461,7 +502,7 @@ class BaseTV(object):
             The size of the current wake lock, or ``None`` if it could not be determined
 
         """
-        wake_lock_size_response = self.adb.shell(constants.CMD_WAKE_LOCK_SIZE)
+        wake_lock_size_response = self._adb.shell(constants.CMD_WAKE_LOCK_SIZE)
 
         return self._wake_lock_size(wake_lock_size_response)
 
@@ -470,6 +511,30 @@ class BaseTV(object):
     #                            Parse properties                             #
     #                                                                         #
     # ======================================================================= #
+    @staticmethod
+    def _audio_output_device(stream_music):
+        """Get the current audio playback device from the ``STREAM_MUSIC`` block from ``adb shell dumpsys audio``.
+
+        Parameters
+        ----------
+        stream_music : str, None
+            The ``STREAM_MUSIC`` block from ``adb shell dumpsys audio``
+
+        Returns
+        -------
+        str, None
+            The current audio playback device, or ``None`` if it could not be determined
+
+        """
+        if not stream_music:
+            return None
+
+        matches = re.findall(constants.DEVICE_REGEX_PATTERN, stream_music, re.DOTALL | re.MULTILINE)
+        if matches:
+            return matches[0]
+
+        return None
+
     @staticmethod
     def _audio_state(audio_state_response):
         """Parse the :attr:`audio_state` property from the output of the command :py:const:`androidtv.constants.CMD_AUDIO_STATE`.
@@ -543,30 +608,6 @@ class BaseTV(object):
 
         return current_app, media_session_state
 
-    @staticmethod
-    def _device(stream_music):
-        """Get the current playback device from the ``STREAM_MUSIC`` block from ``adb shell dumpsys audio``.
-
-        Parameters
-        ----------
-        stream_music : str, None
-            The ``STREAM_MUSIC`` block from ``adb shell dumpsys audio``
-
-        Returns
-        -------
-        str, None
-            The current playback device, or ``None`` if it could not be determined
-
-        """
-        if not stream_music:
-            return None
-
-        matches = re.findall(constants.DEVICE_REGEX_PATTERN, stream_music, re.DOTALL | re.MULTILINE)
-        if matches:
-            return matches[0]
-
-        return None
-
     def _get_stream_music(self, stream_music_raw=None):
         """Get the ``STREAM_MUSIC`` block from the output of the command :py:const:`androidtv.constants.CMD_STREAM_MUSIC`.
 
@@ -582,7 +623,7 @@ class BaseTV(object):
 
         """
         if not stream_music_raw:
-            stream_music_raw = self.adb.shell(constants.CMD_STREAM_MUSIC)
+            stream_music_raw = self._adb.shell(constants.CMD_STREAM_MUSIC)
 
         if not stream_music_raw:
             return None
@@ -665,15 +706,15 @@ class BaseTV(object):
 
         return None
 
-    def _volume(self, stream_music, device):
+    def _volume(self, stream_music, audio_output_device):
         """Get the absolute volume level from the ``STREAM_MUSIC`` block from ``adb shell dumpsys audio``.
 
         Parameters
         ----------
         stream_music : str, None
             The ``STREAM_MUSIC`` block from ``adb shell dumpsys audio``
-        device : str, None
-            The current playback device
+        audio_output_device : str, None
+            The current audio playback device
 
         Returns
         -------
@@ -691,10 +732,10 @@ class BaseTV(object):
             else:
                 self.max_volume = 15.
 
-        if not device:
+        if not audio_output_device:
             return None
 
-        volume_matches = re.findall(device + constants.VOLUME_REGEX_PATTERN, stream_music, re.DOTALL | re.MULTILINE)
+        volume_matches = re.findall(audio_output_device + constants.VOLUME_REGEX_PATTERN, stream_music, re.DOTALL | re.MULTILINE)
         if volume_matches:
             return int(volume_matches[0])
 
@@ -741,11 +782,97 @@ class BaseTV(object):
 
         return None
 
+    # ======================================================================= #
+    #                                                                         #
+    #                               App methods                               #
+    #                                                                         #
+    # ======================================================================= #
+    def _send_intent(self, pkg, intent, count=1):
+        """Send an intent to the device.
+
+        Parameters
+        ----------
+        pkg : str
+            The command that will be sent is ``monkey -p <pkg> -c <intent> <count>; echo $?``
+        intent : str
+            The command that will be sent is ``monkey -p <pkg> -c <intent> <count>; echo $?``
+        count : int, str
+            The command that will be sent is ``monkey -p <pkg> -c <intent> <count>; echo $?``
+
+        Returns
+        -------
+        dict
+            A dictionary with keys ``'output'`` and ``'retcode'``, if they could be determined; otherwise, an empty dictionary
+
+        """
+        cmd = 'monkey -p {} -c {} {}; echo $?'.format(pkg, intent, count)
+
+        # adb shell outputs in weird format, so we cut it into lines,
+        # separate the retcode and return info to the user
+        res = self._adb.shell(cmd)
+        if res is None:
+            return {}
+
+        res = res.strip().split("\r\n")
+        retcode = res[-1]
+        output = "\n".join(res[:-1])
+
+        return {"output": output, "retcode": retcode}
+
+    def launch_app(self, app):
+        """Launch an app.
+
+        Parameters
+        ----------
+        app : str
+            The ID of the app that will be launched
+
+        """
+        self._adb.shell(constants.CMD_LAUNCH_APP.format(app))
+
+    def stop_app(self, app):
+        """Stop an app.
+
+        Parameters
+        ----------
+        app : str
+            The ID of the app that will be stopped
+
+        Returns
+        -------
+        str, None
+            The output of the ``am force-stop`` ADB shell command, or ``None`` if the device is unavailable
+
+        """
+        return self._adb.shell("am force-stop {0}".format(app))
+
+    def start_intent(self, uri):
+        """Start an intent on the device.
+
+        Parameters
+        ----------
+        uri : str
+            The intent that will be sent is ``am start -a android.intent.action.VIEW -d <uri>``
+
+        """
+        self._adb.shell("am start -a android.intent.action.VIEW -d {}".format(uri))
+
     # ======================================================================= #
     #                                                                         #
     #                      "key" methods: basic commands                      #
     #                                                                         #
     # ======================================================================= #
+    def _key(self, key):
+        """Send a key event to device.
+
+        Parameters
+        ----------
+        key : str, int
+            The Key constant
+
+        """
+        self._adb.shell('input keyevent {0}'.format(key))
+
     def power(self):
         """Send power action."""
         self._key(constants.KEY_POWER)
@@ -977,21 +1104,13 @@ class BaseTV(object):
     #                              volume methods                             #
     #                                                                         #
     # ======================================================================= #
-    def set_volume_level(self, volume_level, current_volume_level=None):
+    def set_volume_level(self, volume_level):
         """Set the volume to the desired level.
 
-        .. note::
-
-           This method works by sending volume up/down commands with a 1 second pause in between.  Without this pause,
-           the device will do a quick power cycle.  This is the most robust solution I've found so far.
-
-
         Parameters
         ----------
         volume_level : float
             The new volume level (between 0 and 1)
-        current_volume_level : float, None
-            The current volume level (between 0 and 1); if it is not provided, it will be determined
 
         Returns
         -------
@@ -999,32 +1118,15 @@ class BaseTV(object):
             The new volume level (between 0 and 1), or ``None`` if ``self.max_volume`` could not be determined
 
         """
-        # if necessary, determine the current volume and/or the max volume
-        if current_volume_level is None or not self.max_volume:
-            current_volume = self.volume
-        else:
-            current_volume = min(max(round(self.max_volume * current_volume_level), 0.), self.max_volume)
+        # if necessary, determine the max volume
+        if not self.max_volume:
+            _ = self.volume
+            if not self.max_volume:
+                return None
 
-        # if `self.max_volume` or `current_volume` could not be determined, do not proceed
-        if not self.max_volume or current_volume is None:
-            return None
+        new_volume = int(min(max(round(self.max_volume * volume_level), 0.), self.max_volume))
 
-        new_volume = min(max(round(self.max_volume * volume_level), 0.), self.max_volume)
-
-        # Case 1: the new volume is the same as the current volume
-        if new_volume == current_volume:
-            return new_volume / self.max_volume
-
-        # Case 2: the new volume is less than the current volume
-        if new_volume < current_volume:
-            cmd = "(" + " && sleep 1 && ".join(["input keyevent {0}".format(constants.KEY_VOLUME_DOWN)] * int(current_volume - new_volume)) + ") &"
-
-        # Case 3: the new volume is greater than the current volume
-        else:
-            cmd = "(" + " && sleep 1 && ".join(["input keyevent {0}".format(constants.KEY_VOLUME_UP)] * int(new_volume - current_volume)) + ") &"
-
-        # send the volume down/up commands
-        self.adb.shell(cmd)
+        self._adb.shell("media volume --show --stream 3 --set {}".format(new_volume))
 
         # return the new volume level
         return new_volume / self.max_volume
